package paraphylo.dao;

import java.util.Random;

public class Parameters {

	public static int NO_NOISE = 0;
	public static int HOMOLOGOUS_NOISE = 1;
	public static int ORTHOLOGOUS_NOISE = 2;
	public static int PARALOGOUS_NOISE = 3;
	public static int XENOLOGOUS_NOISE = 4;
	
	public static int NO_BOOTSTRAPPING = 0;
	public static int COGRAPH_BOOTSTRAPPING = 1;
	public static int TRIPLE_BOOTSTRAPPING = 2;
	
	public static int DEFAULT_INPUT = 0;
	public static int COGRAPH_INPUT = 1;
	public static int TRIPLE_INPUT = 2;
	
	private String input;
	private int inputType;
	private String output;
	private int noiseType;
	private double noiseProbability;
	private int cographSize;
	private double cographDistance;
	private boolean cographAllSolutions;
	private boolean write;
	private double enrichment;
	private double enrichmentFactor;
	private int bootstrap;
	private int replications;
	private double support;
	private int timelimitCGE;
	private int timelimitMCS;
	private int timelimitLRT;
	private boolean quickTE;
	private boolean greedyCS;
	private boolean noLTR;
	private boolean verbose;
	private Random random;
	
	public Parameters(Random random) {
		this.input = "";
		this.inputType = 0;
		this.output = "";
		this.noiseType = 0;
		this.noiseProbability = 0d;
		this.cographSize = -1;
		this.cographDistance = 1d;
		this.cographAllSolutions = false;
		this.write = false;
		this.enrichment = 0d;
		this.enrichmentFactor = 0.25d;
		this.bootstrap = 0;
		this.replications = 100;
		this.support = 0;
		this.timelimitCGE = -1;
		this.timelimitMCS = -1;
		this.timelimitLRT = -1;
		this.quickTE = true;
		this.greedyCS = false;
		this.noLTR = false;
		this.verbose = false;
		if (random == null){
			long seed = new Random().nextLong();
			System.out.println("Seed for random number generator is: "+seed);
			random = new Random(seed);
		}
		this.random = random;
	}
	
	public String getInput() {
		return input;
	}

	public void setInput(String input) {
		this.input = input;
	}

	public int getInputType() {
		return inputType;
	}
	
	public void setInputType(int inputType) {
		this.inputType = inputType;
	}

	public String getOutput() {
		return output;
	}

	public void setOutput(String output) {
		this.output = output;
	}

	public int getNoiseType() {
		return noiseType;
	}

	public void setNoiseType(int noiseType) {
		this.noiseType = noiseType;
	}

	public int getCographSize() {
		return cographSize;
	}

	public void setCographSize(int cographSize) {
		this.cographSize = cographSize;
	}

	public double getCographDistance() {
		return cographDistance;
	}

	public void setCographDistance(double cographDistance) {
		this.cographDistance = cographDistance;
	}

	public boolean isCographAllSolutions() {
		return cographAllSolutions;
	}

	public void setCographAllSolutions(boolean cographAllSolutions) {
		this.cographAllSolutions = cographAllSolutions;
	}

	public boolean isWrite() {
		return write;
	}

	public void setWrite(boolean write) {
		this.write = write;
	}

	public double getEnrichment() {
		return enrichment;
	}

	public void setEnrichment(double enrichment) {
		this.enrichment = enrichment;
	}

	public double getEnrichmentFactor() {
		return enrichmentFactor;
	}

	public void setEnrichmentFactor(double enrichmentFactor) {
		this.enrichmentFactor = enrichmentFactor;
	}

	public int getBootstrap() {
		return bootstrap;
	}

	public void setBootstrap(int bootstrap) {
		this.bootstrap = bootstrap;
	}

	public int getReplications() {
		return replications;
	}

	public void setReplications(int replications) {
		this.replications = replications;
	}

	public double getSupport() {
		return support;
	}

	public void setSupport(double support) {
		this.support = support;
	}

	public int getTimelimitCGE() {
		return timelimitCGE;
	}

	public void setTimelimitCGE(int timelimitCGE) {
		this.timelimitCGE = timelimitCGE;
	}

	public int getTimelimitMCS() {
		return timelimitMCS;
	}

	public void setTimelimitMCS(int timelimitMCS) {
		this.timelimitMCS = timelimitMCS;
	}

	public int getTimelimitLRT() {
		return timelimitLRT;
	}

	public void setTimelimitLRT(int timelimitLRT) {
		this.timelimitLRT = timelimitLRT;
	}

	public boolean noiseOrthologs() {
		return (this.noiseType==1 || this.noiseType==2);
	}

	public boolean noiseParalogs() {
		return (this.noiseType==1 || this.noiseType==3);
	}
	
	public boolean noiseAssociations() {
		return (this.noiseType==4);
	}

	public double getNoiseProbability() {
		return noiseProbability;
	}
	
	public void setNoiseProbability(double noiseProbability) {
		this.noiseProbability = noiseProbability;
	}

	public boolean isQuickTE() {
		return quickTE;
	}

	public void setQuickTE(boolean quickTE) {
		this.quickTE = quickTE;
	}
	
	public boolean isGreedyCS() {
		return greedyCS;
	}

	public void setGreedyCS(boolean greedyCS) {
		this.greedyCS = greedyCS;
	}

	public boolean isNoLTR() {
		return noLTR;
	}

	public void setNoLTR(boolean noLTR) {
		this.noLTR = noLTR;
	}

	public boolean isVerbose() {
		return verbose;
	}

	public void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}

	public Random getRandom() {
		return random;
	}

	public void setRandom(Random random) {
		this.random = random;
	}
	
	public boolean nextNoise() {
		return (random.nextDouble()<this.noiseProbability);
	}
}
