package paraphylo.dataset;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import paraphylo.dao.Triple;

public class GeneFamily {

	private int noOfSpecies;
	private int noOfGenes;
	private int noOfSpeciesInGeneFamily;
	private String[] speciesNames;
	private String[] geneSpeciesAssociation;
	private Map<String, Integer> inverseSpeciesIds;
	private String geneTree;
	private String[] geneNames;
	private List<Integer>[] genesOfSpecies;
	private double[][] orthologyMatrix;
	private double[][] bitscores;
	private double[][] evalues;
	private List<Boolean[][]> cographs;
	List<Triple> triples;
	private int cographEditDistance;
	
	public GeneFamily(int noOfSpecies, String[] speciesNames){
		this.noOfSpecies = noOfSpecies;
		this.speciesNames = speciesNames;
		this.geneSpeciesAssociation = new String[0];
		this.noOfGenes = 0;
		this.geneTree = null;
		this.geneNames = null;
		this.orthologyMatrix = new double[0][0];
		this.cographEditDistance = 0;
		
		this.cographs = new ArrayList<Boolean[][]>();
		this.triples = new ArrayList<Triple>();
	}
	
	@SuppressWarnings("unchecked")
	public void init(String[] geneSpeciesAssociation){
		this.geneSpeciesAssociation = geneSpeciesAssociation;
		this.noOfGenes = geneSpeciesAssociation.length;
		this.geneTree = null;
		this.geneNames = null;
		this.orthologyMatrix = new double[noOfGenes][noOfGenes];
		
		this.inverseSpeciesIds = new TreeMap<String, Integer>();
		for (int species=0; species<this.noOfSpecies; species++){
			this.inverseSpeciesIds.put(this.speciesNames[species], species);
		}
		this.genesOfSpecies = new ArrayList[this.noOfSpecies];
		for (int species=0; species<this.noOfSpecies; species++){
			this.genesOfSpecies[species] = new ArrayList<Integer>();
		}
		for (int gene=0; gene<geneSpeciesAssociation.length; gene++){
			int species = this.inverseSpeciesIds.get(geneSpeciesAssociation[gene]);
			this.genesOfSpecies[species].add(gene);
		}
		
		this.noOfSpeciesInGeneFamily = 0;
		for (List<Integer> genes : this.genesOfSpecies){
			if (genes.size()>0){
				this.noOfSpeciesInGeneFamily++;
			}
		}
		
	}
	
	public int getNoOfSpecies() {
		return noOfSpecies;
	}

	public void setNoOfSpecies(int noOfSpecies) {
		this.noOfSpecies = noOfSpecies;
	}

	public int getNoOfGenes() {
		return noOfGenes;
	}

	public void setNoOfGenes(int noOfGenes) {
		this.noOfGenes = noOfGenes;
	}

	public int getNoOfSpeciesInGeneFamily() {
		return noOfSpeciesInGeneFamily;
	}

	public void setNoOfSpeciesInGeneFamily(int noOfSpeciesInGeneFamily) {
		this.noOfSpeciesInGeneFamily = noOfSpeciesInGeneFamily;
	}

	public String[] getSpeciesNames() {
		return speciesNames;
	}

	public String[] getGeneSpeciesAssociation() {
		return geneSpeciesAssociation;
	}

	public void setSpeciesIds(String[] speciesNames) {
		this.speciesNames = speciesNames;
	}

	public String getGeneTree() {
		return geneTree;
	}

	public void setGeneTree(String geneTree) {
		this.geneTree = geneTree;
	}

	public String[] getGeneNames() {
		return geneNames;
	}

	public void setGeneNames(String[] geneNames) {
		this.geneNames = geneNames;
	}

	public List<Integer>[] getGenesOfSpecies() {
		return genesOfSpecies;
	}
	
	public List<Integer> getGenesOfSpecies(int i) {
		if (i<this.noOfSpecies){
			return genesOfSpecies[i];
		}
		else {
			return new ArrayList<Integer>();
		}
	}

	public void setGenesOfSpecies(List<Integer>[] genesOfSpecies) {
		this.genesOfSpecies = genesOfSpecies;
	}

	public double[][] getOrthologyMatrix() {
		return orthologyMatrix;
	}

	public void setOrthologyMatrix(double[][] orthologyMatrix) {
		this.orthologyMatrix = orthologyMatrix;
	}

	public double[][] getBitscores() {
		return bitscores;
	}

	public void setBitscores(double[][] bitscores) {
		this.bitscores = bitscores;
	}

	public double[][] getEvalues() {
		return evalues;
	}

	public void setEvalues(double[][] evalues) {
		this.evalues = evalues;
	}

	public Boolean[][] getCograph(int i) {
		return cographs.get(i);
	}
	
	public List<Boolean[][]> getCographs() {
		return cographs;
	}

	public void addCograph(Boolean[][] cograph) {
		this.cographs.add(cograph);
	}

	public List<Triple> getTriples() {
		return triples;
	}

	public void setTriples(List<Triple> triples) {
		this.triples = triples;
	}

	public int getCographEditDistance() {
		return cographEditDistance;
	}

	public void setCographEditDistance(int cographEditDistance) {
		this.cographEditDistance = cographEditDistance;
	}
	
	public int getSpeciesIdFromGene(int gene){
		for (int species=0; species<this.noOfSpecies; species++){
			if (this.getGeneSpeciesAssociation()[gene].equals(this.speciesNames[species])){
				return species;
			}
		}
		return -1;
	}
	
}
