package paraphylo.dataset;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import paraphylo.dao.Node;
import paraphylo.dao.Triple;
import paraphylo.util.ClosureComputation;
import paraphylo.util.Utils;


public class MultiGeneFamilySpeciesDataSet {

	private int noOfSpecies;
	private String[] speciesNames;
	private String originalTree;
	private String buildTree;
	private String lrTree;
	private List<GeneFamily> geneFamilies;

	double[][][] triples;
	double[][][] enrichedTriples;
	boolean[][][] consistentTriples;
	boolean hirarchy[][];
	
	public MultiGeneFamilySpeciesDataSet(int noOfSpecies, String[] speciesNames){
		this.noOfSpecies = noOfSpecies;
		this.speciesNames = speciesNames;
		this.originalTree = "";
		this.buildTree = getZeroTree();
		this.lrTree = getZeroTree();
		this.geneFamilies = new ArrayList<GeneFamily>();
		this.triples = new double[this.noOfSpecies][this.noOfSpecies][this.noOfSpecies];
		this.consistentTriples = new boolean[this.noOfSpecies][this.noOfSpecies][this.noOfSpecies];
	}
	
	public String getZeroTree(){
		String zeroTree = "(";
		for (String speciesName : speciesNames){
			zeroTree += speciesName+",";
		}
		zeroTree = zeroTree.substring(0, zeroTree.length()-1);
		zeroTree += ");";
		return zeroTree;
	}
	
	public int getNoOfSpecies() {
		return noOfSpecies;
	}

	public void setNoOfSpecies(int noOfSpecies) {
		this.noOfSpecies = noOfSpecies;
	}

	public String[] getSpeciesNames() {
		return speciesNames;
	}

	public void setSpeciesNames(String[] speciesNames) {
		this.speciesNames = speciesNames;
	}

	public String getOriginalTree() {
		return originalTree;
	}

	public void setOriginalTree(String originalTree) {
		this.originalTree = originalTree;
	}

	public String getBuildTree() {
		return buildTree;
	}

	public void setBuildTree(String buildTree) {
		this.buildTree = buildTree;
	}

	public String getLrTree() {
		return lrTree;
	}

	public void setLrTree(String lrTree) {
		this.lrTree = lrTree;
	}

	public List<GeneFamily> getGeneFamilies() {
		return geneFamilies;
	}

	public void setGeneFamilies(List<GeneFamily> geneFamilies) {
		this.geneFamilies = geneFamilies;
	}
	
	public Iterator<GeneFamily> getGeneFamilyIterator(){
		return this.geneFamilies.iterator();
	}
	
	public boolean addGeneFamilies(GeneFamily geneFamily) {
		return geneFamilies.add(geneFamily);
	}

	public double[][][] getTriples() {
		return triples;
	}

	public void setTriples(double[][][] triples) {
		this.triples = triples;
	}
	
	public double getTriple(int A, int B, int C) {
		return triples[A][B][C];
	}
	
	public void setTriple(int A, int B, int C, double value) {
		triples[A][B][C]=value;
		triples[B][A][C]=value;
	}
	
	public void addTriple(int A, int B, int C) {
		if (this.triples==null){
			this.triples=new double[noOfSpecies][noOfSpecies][noOfSpecies];
		}
		if (A!=B && A!=C && B!=C){
			this.triples[A][B][C]++;
			this.triples[B][A][C]++;
		}
	}

	public int getNoOfDistinctTriples(){
		int noOfTriples=0;
		for (int A=0; A<noOfSpecies; A++){
			for (int B=A+1; B<noOfSpecies; B++){
				for (int C=0; C<noOfSpecies; C++){
					if (A!=C && A!=B && this.triples[A][B][C]>0){
						noOfTriples++;
					}
				}
			}
		}
		return noOfTriples;
	}
	
	public double getNoOfTriples(){
		double noOfTriples=0;
		for (int A=0; A<noOfSpecies; A++){
			for (int B=A+1; B<noOfSpecies; B++){
				for (int C=0; C<noOfSpecies; C++){
					if (A!=C && A!=B && this.triples[A][B][C]>0){
						noOfTriples+=this.triples[A][B][C];
					}
				}
			}
		}
		return noOfTriples;
	}
	
	public double[][][] getEnrichedTriples() {
		return enrichedTriples;
	}

	public void setEnrichedTriples(double[][][] enrichedTriples) {
		this.enrichedTriples = enrichedTriples;
	}
	
	public void addEnrichedTriple(int A, int B, int C) {
		if (this.enrichedTriples==null){
			this.enrichedTriples=new double[noOfSpecies][noOfSpecies][noOfSpecies];
		}
		if (A!=B && A!=C && B!=C){
			this.enrichedTriples[A][B][C]++;
			this.enrichedTriples[B][A][C]++;
		}
	}
	
	public double getNoOfEnrichedTriples(){
		double noOfTriples=0;
		if (this.enrichedTriples!=null){
			for (int A=0; A<noOfSpecies; A++){
				for (int B=A+1; B<noOfSpecies; B++){
					for (int C=0; C<noOfSpecies; C++){
						if (A!=C && A!=B && this.enrichedTriples[A][B][C]>0){
							noOfTriples+=this.enrichedTriples[A][B][C];
						}
					}
				}
			}
		}
		return noOfTriples;
	}

	public double getSupport(int a, int b, int c){
		double sum = this.triples[a<b?a:b][a<b?b:a][c] + this.triples[a<c?a:c][a<c?c:a][b] + this.triples[b<c?b:c][b<c?c:b][a];
		if (sum>0){
			return this.triples[a<b?a:b][a<b?b:a][c]/sum;
		}
		return 0d;
	}

	public boolean[][][] getConsistentTriples() {
		return consistentTriples;
	}

	public void setConsistentTriples(boolean[][][] consistentTriples) {
		this.consistentTriples = consistentTriples;
	}
	
	public boolean getConsistentTriple(int A, int B, int C) {
		return consistentTriples[A][B][C];
	}
	
	public void setCosistentTriple(int A, int B, int C, boolean value) {
		consistentTriples[A][B][C]=value;
	}
	
	public int getNoOfConsistentTriples(){
		int noOfTriples=0;
		for (int A=0; A<noOfSpecies; A++){
			for (int B=A+1; B<noOfSpecies; B++){
				for (int C=0; C<noOfSpecies; C++){
					if (A!=C && A!=B && this.consistentTriples[A][B][C]){
						noOfTriples++;
					}
				}
			}
		}
		return noOfTriples;
	}
	
	public void closeConsistentTriples(){
		List<Triple> tripleList = new ArrayList<Triple>();
		for (int A=0; A<noOfSpecies; A++){
			for (int B=A+1; B<noOfSpecies; B++){
				for (int C=0; C<noOfSpecies; C++){
					if (A!=C && A!=B && this.consistentTriples[A][B][C]){
						tripleList.add(new Triple(A,B,C));
					}
				}
			}
		}
		tripleList = ClosureComputation.getClosure(tripleList);
		for (Triple triple : tripleList){
			setCosistentTriple(triple.getA(),triple.getB(),triple.getC(),true);
		}
	}
	
	public boolean[][] getHirarchy() {
		return hirarchy;
	}

	public void setHirarchy(boolean[][] hirarchy) {
		this.hirarchy = hirarchy;
	}

	public String computeLRTree(){
		if (this.hirarchy==null){
			return this.buildTree;
		}
		List<Integer> childIds = new ArrayList<Integer>();
		for (int i=0; i<this.noOfSpecies; i++){
			childIds.add(i);
		}
		Node root = new Node(null, childIds);
		String value = String.valueOf(Utils.computeSupport(this));
		root.setValue(value);
		List<Node> nodes = new ArrayList<Node>();
		nodes.add(root);
		for (int p=0; p<this.noOfSpecies-2; p++){
			childIds = new ArrayList<Integer>();
			for (int a=0; a<this.noOfSpecies;a++){
				if (hirarchy[a][p]){
					childIds.add(a);
				}
			}
			if (childIds.size()>1 && childIds.size()<this.noOfSpecies){
				Iterator<Node> iter = nodes.iterator();
				Node child = new Node(null, childIds);
				value = String.valueOf(Utils.computeSubtreeSupport(childIds, this.triples, this.noOfSpecies));
				child.setValue(value);
				while (iter.hasNext()){
					Node node = iter.next();
					if (node.getChildIds().contains(childIds.get(0))){
						child.setParent(node);
						node.getChildren().add(child);
						break;
					}
				}
				nodes.add(0, child);
			}
		}
		for (Node node : nodes){
			for (int i : node.getChildIds()){
				boolean found = false;
				for (Node child : node.getChildren()){
					if (child.getChildIds().contains(i)){
						found = true;
						break;
					}
				}
				if (!found){
					childIds = new ArrayList<Integer>();
					childIds.add(i);
					Node child = new Node(node, childIds);
					child.setName(String.valueOf(speciesNames[i]));
					node.getChildren().add(child);
				}
			}
		}
		this.lrTree = root.toString()+";";
		return this.lrTree;
	}

	public String getCographOutput() {
		String output = "";
		output += "#NoOfSpecies\n";
		output += this.noOfSpecies+"\n";
		output += "#SpeciesNames\n";
		for (String speciesName : this.speciesNames){
			output += speciesName + "\t";
		}
		output += "\n\n";
		int counter = 1;
		for (GeneFamily family : geneFamilies){
			String gfoutput = "#GeneFamily "+counter+"\n";
			counter++;
			gfoutput += "#GeneSpeciesAssociations\n";
			for (int i=0; i<family.getNoOfGenes(); i++){
				gfoutput += family.getGeneSpeciesAssociation()[i]+"\t";
			}
			gfoutput += "\n";
			for (Boolean[][] cograph : family.getCographs()){
				gfoutput += "#Cograph\n";
				for (int i=0; i<family.getNoOfGenes(); i++){
					for (int j=0; j<family.getNoOfGenes(); j++){
						gfoutput += cograph[i][j].booleanValue()?"1\t":"0\t";
					}
					gfoutput+="\n";
				}
				gfoutput += "\n";
			}
			output += gfoutput;
		}
		
		return output;
	}

	public String getTripleOutput() {
		String output = "";
		output += "#NoOfSpecies\n";
		output += this.noOfSpecies+"\n";
		output += "#SpeciesNames\n";
		for (String speciesName : this.speciesNames){
			output += speciesName + "\t";
		}
		output += "\n\n";
		output += "#Triples\n";
		for (int A=0; A<noOfSpecies; A++){
			for (int B=A+1; B<noOfSpecies; B++){
				for (int C=0; C<noOfSpecies; C++){
					if (A!=C && A!=B && this.triples[A][B][C]>0){
						output += A+","+B+"|"+C+"\t"+this.triples[A][B][C]+"\n";
					}
				}
			}
		}
		return output;
	}

	public List<Triple> getTripleList() {
		List<Triple> triples = new ArrayList<Triple>();
		for (int A=0; A<noOfSpecies; A++){
			for (int B=A+1; B<noOfSpecies; B++){
				for (int C=0; C<noOfSpecies; C++){
					if (A!=C && A!=B){
						for (int i=0; i<this.triples[A][B][C]; i++){
							triples.add(new Triple(A, B, C));
						}
					}
				}
			}
		}
		return triples;
	}
	
}
