package paraphylo.solver.cplex;

import ilog.concert.IloException;
import ilog.concert.IloNumExpr;
import ilog.cplex.IloCplex;

import java.io.File;
import java.lang.reflect.Field;

import paraphylo.dao.Parameters;
import paraphylo.dataset.MultiGeneFamilySpeciesDataSet;

public class CplexAbstractSolver {

	static {
		try {
			String workdir = new File(".").getCanonicalPath();
			if (System.getProperty("os.name").toLowerCase().contains("windows")) {
				// try to load Windows library
				if (System.getProperty("os.arch").toLowerCase().contains("64")) {
					System.setProperty("java.library.path", System.getProperty("java.library.path")+":"+workdir+"/os/windows/x86_64");
				}
				else if (System.getProperty("os.arch").toLowerCase().contains("86")) {
					System.setProperty("java.library.path", System.getProperty("java.library.path")+":"+workdir+"/os/windows/x86");
				}
				else {
					System.out.println("The os and/or arch is not supported.");
				}
				
			}
			else if (System.getProperty("os.name").toLowerCase().contains("linux")) {
				// try to load Linux library
				if (System.getProperty("os.arch").toLowerCase().contains("64")) {
					System.setProperty("java.library.path", System.getProperty("java.library.path")+":"+workdir+"/os/linux/x86_64");
				}
				else if (System.getProperty("os.arch").toLowerCase().contains("86")) {
					System.setProperty("java.library.path", System.getProperty("java.library.path")+":"+workdir+"/os/linux/x86");
				}
				else {
					System.out.println("The os and/or arch is not supported.");
				}
			}
			else if (System.getProperty("os.name").toLowerCase().contains("mac")) {
				// try to load Mac library
				if (System.getProperty("os.arch").toLowerCase().contains("64")) {
					System.setProperty("java.library.path", System.getProperty("java.library.path")+":"+workdir+"/os/macosx/x86_64");
				}
				else if (System.getProperty("os.arch").toLowerCase().contains("86")) {
					System.setProperty("java.library.path", System.getProperty("java.library.path")+":"+workdir+"/os/macosx/x86");
				}
				else {
					System.out.println("The os and/or arch is not supported.");
				}
			} 
			else {
				System.out.println("The os and/or arch is not supported.");
			}
			Field fieldSysPath = ClassLoader.class.getDeclaredField( "sys_paths" );
			fieldSysPath.setAccessible( true );
			fieldSysPath.set( null, null );
			System.loadLibrary("cplex1260");
		} catch (Exception e) {
			System.out.println("The dynamic link library for CPLEX could not be"
									+ "loaded.\nConsider using\njava -Djava.library.path=");
			System.exit(1);
		}
	}
	
	// input data
	MultiGeneFamilySpeciesDataSet ds;
	// parameters
	Parameters parameters;
	// number of species
	int noOfSpecies;
	// CPLEX solver
	IloCplex solver;
	// objective function
	IloNumExpr objFn;
	
	public CplexAbstractSolver(MultiGeneFamilySpeciesDataSet ds, Parameters parameters) throws IloException{ 
		this.ds = ds;
		this.noOfSpecies = this.ds.getNoOfSpecies();
		this.parameters = parameters;
		this.solver = new IloCplex();
	}
	
}
