package paraphylo.solver.cplex;

import ilog.concert.IloException;
import ilog.concert.IloIntVar;
import ilog.cplex.IloCplex.UnknownObjectException;
import paraphylo.dao.Parameters;
import paraphylo.dataset.MultiGeneFamilySpeciesDataSet;
import paraphylo.util.NullOutputStream;


public class CplexLeastResolvedTreeSizeLowerBoundSolver extends CplexAbstractSolver{
	
	// partial order variables
	private IloIntVar[] X;
	// left right order variables
	private IloIntVar[][][] LR;
	// max order variable
	private IloIntVar maxX;
	
	public CplexLeastResolvedTreeSizeLowerBoundSolver(MultiGeneFamilySpeciesDataSet ds, Parameters parameters) throws IloException{
		super(ds, parameters);
	}
	
	public int solve() throws IloException{
		this.solver.setName("CplexLeastResolvedTreeSizeLowerBoundSolver");
		
		int[] lb = new int[this.noOfSpecies];
		int[] ub = new int[this.noOfSpecies];
		for (int i=0; i<this.noOfSpecies; i++){
			lb[i]=0;
			ub[i]=this.noOfSpecies-1;
		}
		this.X = solver.intVarArray(this.noOfSpecies, lb, ub);
		this.maxX = solver.intVar(0, this.noOfSpecies-1);
		for (int i=0; i<this.noOfSpecies; i++){
			this.X[i].setName("X_"+i);
			solver.addLe(this.X[i], this.maxX);
		}
		
		this.LR = new IloIntVar[noOfSpecies][noOfSpecies][noOfSpecies];
		for (int a=0; a<noOfSpecies; a++){
			for (int b=a+1; b<noOfSpecies; b++){
				for (int c=b+1; c<noOfSpecies; c++){
					if (ds.getConsistentTriple(a, b, c)){
						LR[a][b][c] = solver.boolVar("LR_("+a+","+b+"|"+c+")");
						solver.addLe(this.X[a], solver.sum(solver.sum(this.X[c], -1), solver.prod(this.noOfSpecies, solver.diff(1, LR[a][b][c]))));
						solver.addLe(this.X[b], solver.sum(solver.sum(this.X[c], -1), solver.prod(this.noOfSpecies, solver.diff(1, LR[a][b][c]))));
						solver.addLe(this.X[c], solver.sum(solver.sum(this.X[a], -1), solver.prod(this.noOfSpecies, LR[a][b][c])));
						solver.addLe(this.X[c], solver.sum(solver.sum(this.X[b], -1), solver.prod(this.noOfSpecies, LR[a][b][c])));
					}
					if (ds.getConsistentTriple(a, c, b)){
						LR[a][c][b] = solver.boolVar("LR_("+a+","+c+"|"+b+")");
						solver.addLe(this.X[a], solver.sum(solver.sum(this.X[b], -1), solver.prod(this.noOfSpecies, solver.diff(1, LR[a][c][b]))));
						solver.addLe(this.X[c], solver.sum(solver.sum(this.X[b], -1), solver.prod(this.noOfSpecies, solver.diff(1, LR[a][c][b]))));
						solver.addLe(this.X[b], solver.sum(solver.sum(this.X[a], -1), solver.prod(this.noOfSpecies, LR[a][c][b])));
						solver.addLe(this.X[b], solver.sum(solver.sum(this.X[c], -1), solver.prod(this.noOfSpecies, LR[a][c][b])));
					}
					if (ds.getConsistentTriple(b, c, a)){
						LR[b][c][a] = solver.boolVar("LR_("+b+","+c+"|"+a+")");
						solver.addLe(this.X[b], solver.sum(solver.sum(this.X[a], -1), solver.prod(this.noOfSpecies, solver.diff(1, LR[b][c][a]))));
						solver.addLe(this.X[c], solver.sum(solver.sum(this.X[a], -1), solver.prod(this.noOfSpecies, solver.diff(1, LR[b][c][a]))));
						solver.addLe(this.X[a], solver.sum(solver.sum(this.X[b], -1), solver.prod(this.noOfSpecies, LR[b][c][a])));
						solver.addLe(this.X[a], solver.sum(solver.sum(this.X[c], -1), solver.prod(this.noOfSpecies, LR[b][c][a])));
					}
				}
			}
		}
		
		solver.addMinimize(maxX);

		// solve
		if (!parameters.isVerbose()){
			solver.setOut(new NullOutputStream());
		}
		boolean ret = solver.solve();
		int solution = 0;
		if (ret) {
			solution = getSolution();
		}
		// free memory
		solver.end();
		
		return solution;

	}
	
	/**
	 * @return the current solution, i.e. the values of xVariables as string
	 * @throws UnknownObjectException
	 * @throws IloException
	 */
	private int getSolution() throws UnknownObjectException, IloException {
		int solution = (int)Math.floor(solver.getObjValue());
		return solution;
	}
	
}