package paraphylo.solver.greedy;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import paraphylo.dao.Parameters;
import paraphylo.dao.Triple;
import paraphylo.dataset.MultiGeneFamilySpeciesDataSet;
import paraphylo.util.BuildAlgorithm;

public class GreedyConsistentSubsetSolver {
	
	// input data
	MultiGeneFamilySpeciesDataSet ds;
	// parameters
	Parameters parameters;
	// number of species
	int noOfSpecies;
	// consistent triples
	List<Triple> consistentTriples;
	
	public GreedyConsistentSubsetSolver(MultiGeneFamilySpeciesDataSet ds, Parameters parameters) { 
		this.ds = ds;
		this.noOfSpecies = this.ds.getNoOfSpecies();
		this.parameters = parameters;
	}

	public String solve() {
		this.consistentTriples = new ArrayList<Triple>();
		List<Triple> triples = new ArrayList<Triple>();
		
		for (int a=0; a<noOfSpecies; a++){
			for (int b=a+1; b<noOfSpecies; b++){
				for (int c=0; c<noOfSpecies; c++){
					if (a!=c && b!=c){
						if (ds.getTriple(a, b, c)>0){
							if (this.parameters.getSupport()==0 || ds.getSupport(a, b, c) > this.parameters.getSupport()){
								triples.add(new Triple(a,b,c,ds.getTriple(a, b, c)));
							}
						}
					}
				}
			}
		}
		
		// check sorting direction
		Collections.sort(triples);
		
		for(Triple triple : triples){
			consistentTriples.add(0, triple);
			if (!BuildAlgorithm.checkConsistency(consistentTriples)){
				consistentTriples.remove(0);
			}
		}
		
		return getSolution();
	}
	
	private String getSolution() {
		String solution="Triples:\n";
		double objValue = 0;
		int i=0;
		for(Triple triple : this.consistentTriples){
			int a = triple.getA();
			int b = triple.getB();
			int c = triple.getC();
			solution+=a+","+b+"|"+c+"\n";
			ds.setCosistentTriple(a, b, c, true);
			i++;
		}
		solution+=i+" triples\n";
		solution+="\n\n";
		solution+=objValue+"\n";
		return solution;
	}
	
}
