package paraphylo.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import paraphylo.dao.Triple;


public class BuildAlgorithm {

	public static String getTree(List<Triple> triples, Set<Integer> leaves, String[] speciesNames, double[][][] allTriples) throws Exception{
		String speciesTree = "";
		Set<Set<Integer>> sets = new HashSet<Set<Integer>>();
		for (Integer leaf : leaves){
			Set<Integer> s = new HashSet<Integer>();
			s.add(leaf);
			sets.add(s);
		}
		for (Triple triple : triples){
			Set<Integer> set1 = new HashSet<Integer>();
			set1.add(triple.getA());
			set1.add(triple.getB());
			for (Set<Integer> s : sets){
				if (s.contains(triple.getA())){
					set1.addAll(s);
					sets.remove(s);
					break;
				}
			}
			for (Set<Integer> s : sets){
				if (s.contains(triple.getB())){
					set1.addAll(s);
					sets.remove(s);
					break;
				}
			}
			sets.add(set1);
			boolean found2 = false;
			for (Set<Integer> s : sets){
				if (s.contains(triple.getC())){
					found2=true;
					break;
				}
			}
			if (!found2){
				Set<Integer> set2 = new HashSet<Integer>();
				set2.add(triple.getC());
				sets.add(set2);
			}
		}
		if (sets.size()<2){
			throw new Exception("no tree exists");
		}
		speciesTree += "(";
		int i = 0;
		
		for (Set<Integer> set : sets){
			if (i!=0){
				speciesTree += ",";
			}
			if (set.size()==1){
				Iterator<Integer> iter = set.iterator();
				speciesTree += speciesNames[iter.next()];
			}
			else{
				List<Triple> newTriples = new ArrayList<Triple>();
				for (Triple triple : triples){
					if (set.contains(triple.getA()) && set.contains(triple.getB()) && set.contains(triple.getC())){
						newTriples.add(triple);
					}
				}
				if (newTriples.isEmpty()){
					speciesTree += "(";
					int j=0;
					for (int val : set){
						if (j!=0){
							speciesTree += ",";
						}
						speciesTree += speciesNames[val];
						j++;
					}
					speciesTree += ")";
				} else {
					speciesTree += getTree(newTriples, set, speciesNames, allTriples);
				}
				if (set.size()>1){
					speciesTree += String.valueOf(Utils.computeSubtreeSupport(set, allTriples, speciesNames.length));
				}
			}
			i++;
		}
		speciesTree += ")";
		return speciesTree;
	}
	
	
	public static boolean checkConsistency(List<Triple> triples){
		Set<Set<Integer>> sets = new HashSet<Set<Integer>>();
		for (Triple triple : triples){
			Set<Integer> set1 = new HashSet<Integer>();
			set1.add(triple.getA());
			set1.add(triple.getB());
			for (Set<Integer> s : sets){
				if (s.contains(triple.getA())){
					set1.addAll(s);
					sets.remove(s);
					break;
				}
			}
			for (Set<Integer> s : sets){
				if (s.contains(triple.getB())){
					set1.addAll(s);
					sets.remove(s);
					break;
				}
			}
			sets.add(set1);
			boolean found2 = false;
			for (Set<Integer> s : sets){
				if (s.contains(triple.getC())){
					found2=true;
					break;
				}
			}
			if (!found2){
				Set<Integer> set2 = new HashSet<Integer>();
				set2.add(triple.getC());
				sets.add(set2);
			}
		}
		if (sets.size()<2){
			return false;
		}
		
		for (Set<Integer> set : sets){
			if (set.size()>1){
				List<Triple> newTriples = new ArrayList<Triple>();
				for (Triple triple : triples){
					if (set.contains(triple.getA()) && set.contains(triple.getB()) && set.contains(triple.getC())){
						newTriples.add(triple);
					}
				}
				if (!newTriples.isEmpty()){
					if (!checkConsistency(newTriples)){
						return false;
					}
				}
			}
		}
		return true;
	}

	
}
