package paraphylo.util;

import java.util.List;

import paraphylo.dao.Parameters;
import paraphylo.dao.Triple;
import paraphylo.dataset.GeneFamily;
import paraphylo.dataset.MultiGeneFamilySpeciesDataSet;


public class TripleExtraction {
	
	//input data
	MultiGeneFamilySpeciesDataSet ds;
	// parameters
	Parameters parameters;
	// number of species
	int noOfSpecies;
	// gene family
	private GeneFamily gf;
	
	private Boolean[][] cograph;
	private boolean[][][] speciesTriple;
	
	public TripleExtraction(MultiGeneFamilySpeciesDataSet ds, GeneFamily gf, Boolean[][] cograph, Parameters parameters) {
		this.ds = ds;
		this.noOfSpecies = this.ds.getNoOfSpecies();
		this.parameters = parameters;
		this.gf = gf;
		this.cograph = cograph;
		this.speciesTriple = new boolean[this.noOfSpecies][this.noOfSpecies][this.noOfSpecies];
	}
	
	public String solve(){
		for (int x=0; x<gf.getNoOfGenes(); x++){
			for (int y=x+1; y<gf.getNoOfGenes(); y++){
				for (int z=0; z<gf.getNoOfGenes(); z++){
					if (x!=z && y!=z && isDistinctSpecies(x,y,z)){
						if (this.cograph[x][z] && this.cograph[y][z])
						{
							int A = this.gf.getSpeciesIdFromGene(x);
							int B = this.gf.getSpeciesIdFromGene(y);
							int C = this.gf.getSpeciesIdFromGene(z);
							//case 1
							if (!this.cograph[x][y]){
								this.speciesTriple[A][B][C]=true;
								this.speciesTriple[B][A][C]=true;
							}
							else {
								for (int w=0; w<gf.getNoOfGenes(); w++){
									if (x!=w && y!=w && z!=w && this.cograph[z][w] && !this.cograph[x][w] && !this.cograph[y][w]){
										this.speciesTriple[A][B][C]=true;
										this.speciesTriple[B][A][C]=true;
										break;
									}
								}
							}
						}
					}
					
				}
			}
		}
		String solution = getSolution();
		
		return solution;

	}
	
	private boolean isDistinctSpecies(int a, int b, int c) {
		if (this.gf.getGeneSpeciesAssociation()[a].equals(this.gf.getGeneSpeciesAssociation()[b]) ||
				this.gf.getGeneSpeciesAssociation()[a].equals(this.gf.getGeneSpeciesAssociation()[c]) ||
				this.gf.getGeneSpeciesAssociation()[b].equals(this.gf.getGeneSpeciesAssociation()[c])){
			return false;
		}
		return true;
	}

	private boolean isTriple(int A, int B, int C){
		return this.speciesTriple[A][B][C];
	}
	

	/**
	 * @return the current solution, i.e. the values of xVariables as string
	 */
	private String getSolution() {
		List<Triple> triples = gf.getTriples();
		String solution="Triples:\n";
		int i=0;
		for (int A=0; A<this.noOfSpecies;A++){
			for (int B=A+1; B<this.noOfSpecies; B++){
				for (int C=0; C<this.noOfSpecies; C++){
					if (A!=C && B!=C){
						if (isTriple(A,B,C)){
							solution+=A+","+B+"|"+C+"\n";
							ds.addTriple(A, B, C);
							triples.add(new Triple(A,B,C));
							i++;
						}
					}
				}
			}
		}
		solution+=i+" triples\n";
		return solution;
	}
	
}
